@@ -65,7 +65,10 @@ int main(void)
         gnutls_server_name_set(session, GNUTLS_NAME_DNS, "my_host_name",
                                strlen("my_host_name"));
 
-        /* Use default priorities */
+        /* use default priorities */
+        gnutls_set_default_priority(session);
+#if 0
+	/* if more fine-graned control is required */
         ret = gnutls_priority_set_direct(session, 
                                          "NORMAL", &err);
         if (ret < 0) {
@@ -74,6 +77,7 @@ int main(void)
                 }
                 exit(1);
         }
+#endif
 
         /* put the x509 credentials to the current session
          */
