@@ -30,10 +30,11 @@
 #include <gnutls/x509.h>
 #include <c-ctype.h>
 
+#define MAX_ELEMENTS 64
+
 static void
-break_comma_list(char *etag,
-		 char **broken_etag, int *elements, int max_elements,
-		 char sep);
+break_list(char *etag,
+		 char *broken_etag[MAX_ELEMENTS], int *size);
 
 /**
  * gnutls_cipher_set_priority:
@@ -198,7 +199,8 @@ int gnutls_protocol_set_priority(gnutls_session_t session, const int *list)
 		/* set the current version to the first in the chain.
 		 * This will be overridden later.
 		 */
-		_gnutls_set_current_version(session, list[0]);
+		if (_gnutls_set_current_version(session, list[0]) < 0)
+			return gnutls_assert_val(GNUTLS_E_UNSUPPORTED_VERSION_PACKET);
 	}
 
 	return 0;
@@ -614,10 +616,13 @@ gnutls_priority_set(gnutls_session_t session, gnutls_priority_t priority)
 	/* set the current version to the first in the chain.
 	 * This will be overridden later.
 	 */
-	if (session->internals.priorities.protocol.algorithms > 0)
-		_gnutls_set_current_version(session,
+	if (session->internals.priorities.protocol.algorithms > 0) {
+		if (_gnutls_set_current_version(session,
 					    session->internals.priorities.
-					    protocol.priority[0]);
+					    protocol.priority[0]) < 0) {
+			return gnutls_assert_val(GNUTLS_E_UNSUPPORTED_VERSION_PACKET);
+		}
+	}
 
 	if (session->internals.priorities.protocol.algorithms == 0 ||
 	    session->internals.priorities.cipher.algorithms == 0 ||
@@ -630,8 +635,6 @@ gnutls_priority_set(gnutls_session_t session, gnutls_priority_t priority)
 }
 
 
-#define MAX_ELEMENTS 48
-
 #define LEVEL_NONE "NONE"
 #define LEVEL_NORMAL "NORMAL"
 #define LEVEL_PFS "PFS"
@@ -1135,8 +1138,7 @@ gnutls_priority_init(gnutls_priority_t * priority_cache,
 		goto error;
 	}
 
-	break_comma_list(darg, broken_list, &broken_list_size,
-			 MAX_ELEMENTS, ':');
+	break_list(darg, broken_list, &broken_list_size);
 	/* This is our default set of protocol version, certificate types and
 	 * compression methods.
 	 */
@@ -1362,6 +1364,8 @@ void gnutls_priority_deinit(gnutls_priority_t priority_cache)
  * priority cache and is used to directly set string priorities to a
  * TLS session.  For documentation check the gnutls_priority_init().
  *
+ * To simply use a reasonable default, consider using gnutls_set_default_priority().
+ *
  * Returns: On syntax error %GNUTLS_E_INVALID_REQUEST is returned,
  * %GNUTLS_E_SUCCESS on success, or an error code.
  **/
@@ -1393,22 +1397,19 @@ gnutls_priority_set_direct(gnutls_session_t session,
  * MAX_COMMA_SEP_ELEMENTS size; Note that the given string is modified.
   */
 static void
-break_comma_list(char *etag,
-		 char **broken_etag, int *elements, int max_elements,
-		 char sep)
+break_list(char *list,
+		 char *broken_list[MAX_ELEMENTS], int *size)
 {
-	char *p = etag;
-	if (sep == 0)
-		sep = ',';
+	char *p = list;
 
-	*elements = 0;
+	*size = 0;
 
 	do {
-		broken_etag[*elements] = p;
+		broken_list[*size] = p;
 
-		(*elements)++;
+		(*size)++;
 
-		p = strchr(p, sep);
+		p = strchr(p, ':');
 		if (p) {
 			*p = 0;
 			p++;	/* move to next entry and skip white
@@ -1418,7 +1419,7 @@ break_comma_list(char *etag,
 				p++;
 		}
 	}
-	while (p != NULL && *elements < max_elements);
+	while (p != NULL && *size < MAX_ELEMENTS);
 }
 
 /**
@@ -1428,9 +1429,8 @@ break_comma_list(char *etag,
  * Sets some default priority on the ciphers, key exchange methods,
  * macs and compression methods.
  *
- * This is the same as calling:
- *
- * gnutls_priority_set_direct (session, "NORMAL", NULL);
+ * This typically sets a default priority that is considered
+ * sufficiently secure to establish encrypted sessions.
  *
  * This function is kept around for backwards compatibility, but
  * because of its wide use it is still fully supported.  If you wish
